/**
 * Copyright © 2014-2021 The SiteWhere Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.sitewhere.labels.configuration.generator;

import com.fasterxml.jackson.databind.JsonNode;
import com.sitewhere.labels.configuration.manager.LabelGeneratorGenericConfiguration;
import com.sitewhere.microservice.configuration.json.JsonConfiguration;
import com.sitewhere.spi.SiteWhereException;
import com.sitewhere.spi.microservice.lifecycle.ITenantEngineLifecycleComponent;

/**
 * Base class for common label generator configuration.
 */
public abstract class LabelGeneratorConfiguration extends JsonConfiguration {

    /** Unique generator id */
    private String id;

    /** Generator name */
    private String name;

    public LabelGeneratorConfiguration(ITenantEngineLifecycleComponent component) {
	super(component);
    }

    public void apply(LabelGeneratorGenericConfiguration configuration) throws SiteWhereException {
	this.id = configuration.getId();
	this.name = configuration.getName();
	loadFrom(configuration.getConfiguration());
    }

    /**
     * Load subclass from JSON.
     * 
     * @param json
     * @throws SiteWhereException
     */
    public abstract void loadFrom(JsonNode json) throws SiteWhereException;

    public String getId() {
	return id;
    }

    public void setId(String id) {
	this.id = id;
    }

    public String getName() {
	return name;
    }

    public void setName(String name) {
	this.name = name;
    }
}
